@@ -10,7 +10,7 @@
 #' @param var_names Names for the design factors. Must have the same length asfactor.level. Defaults to "X1", "X2", ...
 #' @param xlist_fix list of discrete factor experimental settings under consideration, default NULL indicating a list of all possible discrete factor experimental settings will be used.
 #' @param hfunc function for generating the corresponding model matrix or predictor vector, given an experimental setting or design point.
-#' @param h.prime function to obtain dX/dx
+#' @param h.prime User-supplied derivative function for continuous factors x_(1) in R^k (k is the number of continuous variables). For MLMs: returns a list of J*p matrices dX_x/dx_j, j=1,...,k (numerical derivatives if omitted).  For GLMs: returns the p times k matrix dh(x)/dx_(1) (defaults to main effects if omitted).
 #' @param bvec_matrix Matrix of bootstrapped or simulated parameter values.
 #' @param link link function, default "continuation", other choices "baseline", "cumulative", and "adjacent"
 #' @param EW_Fi.func function to calculate entry wise expectation of Fisher information Fi, default EW_Fi_MLM_func.
@@ -23,6 +23,7 @@
 #' @param nram number of times repeating the lift-one step with random initial allocations, valid only if random is TRUE, default 3.
 #' @param rowmax maximum number of points in the initial design, default NULL indicates no restriction
 #' @param Xini initial list of design points, default NULL indicating automatically generating an initial list of design points.
+#' @param pini A numeric vector specifying the initial weights for the design points in Xini, default NULL indicating automatically generating an uniform weights of design points.
 #' @param random.initial TRUE or FALSE, whether or not to repeat the whole procedure multiple times with random initial designs, default FALSE.
 #' @param nram.initial number of times repeating the whole procedure with random initial designs, valid only if random.initial is TRUE, default 3.
 #' @param optim_grad TRUE or FALSE, default is FALSE, whether to use the analytical gradient function or numerical gradient when searching for a new design point.
@@ -58,14 +59,14 @@
 #'                          -9.2154, -9.7576, -9.6818, -8.5139, -8.56),nrow=4,byrow=TRUE)
 #' EW_ForLion_MLM_Optimal(J=J, n.factor=n.factor.temp, factor.level=factor.level.temp,
 #'          xlist_fix=NULL, hfunc=hfunc.temp,h.prime=h.prime.temp, bvec_matrix=bvec_bootstrap,
-#'          delta=1, link=link.temp, optim_grad=FALSE)
+#'          reltol=1e-2, delta=1, link=link.temp, optim_grad=FALSE)
 #'
 
 
 
 
 
-EW_ForLion_MLM_Optimal <- function(J ,n.factor, factor.level,var_names=NULL,xlist_fix=NULL, hfunc, h.prime, bvec_matrix, link="continuation", EW_Fi.func=EW_Fi_MLM_func, delta0=1e-5, epsilon=1e-12, reltol=1e-5, delta=0, maxit=100, random=FALSE, nram=3, rowmax=NULL, Xini=NULL, random.initial=FALSE, nram.initial=3, optim_grad=FALSE) {
+EW_ForLion_MLM_Optimal <- function(J ,n.factor, factor.level,var_names=NULL,xlist_fix=NULL, hfunc, h.prime, bvec_matrix, link="continuation", EW_Fi.func=EW_Fi_MLM_func, delta0=1e-5, epsilon=1e-12, reltol=1e-5, delta=0, maxit=100, random=FALSE, nram=3, rowmax=NULL, Xini=NULL, pini=NULL, random.initial=FALSE, nram.initial=3, optim_grad=FALSE) {
   d.factor=length(n.factor);             # number of factors
   p.factor=dim(bvec_matrix)[2];                 # number of predictors
   k.continuous=sum(n.factor==0);         # number of continuous factors
@@ -82,8 +83,13 @@ EW_ForLion_MLM_Optimal <- function(J ,n.factor, factor.level,var_names=NULL,xlis
       X.mat[,,i]=htemp;
     };
 
-    p0= rep(1/m.design, m.design)
+    if(is.null(pini)){
+      p0 = rep(1/m.design, m.design)
+    }else{
+      p0 = pini
+    }
     p0 = p0/sum(p0)
+
     optemp=EW_liftoneDoptimal_MLM_func(m=m.design, p=p.factor, Xi=X.mat, J=J, thetavec_matrix=bvec_matrix, link=link, reltol=reltol, maxit=maxit, p00=p0, random=random, nram=nram);
 
     m.design.ans=m.design=sum(optemp$p>0);       # updated number of design point
@@ -99,7 +105,7 @@ EW_ForLion_MLM_Optimal <- function(J ,n.factor, factor.level,var_names=NULL,xlis
   if(k.continuous==d.factor) {
     lvec=uvec=rep(0, d.factor);     # lower bounds and upper bounds for continuous factors
     for(i in 1:d.factor) {lvec[i]=min(factor.level[[i]]); uvec[i]=max(factor.level[[i]]);};
-    if(is.null(Xini)){initial.temp=EW_design_initial_MLM(k.continuous=k.continuous, factor.level=factor.level, xlist_fix=xlist_fix, lvec=lvec, uvec=uvec, bvec_matrix=bvec_matrix, link=link, h.func=hfunc, EW_Fi.func=EW_Fi.func, delta0=delta0, epsilon = epsilon, maxit=maxit); xtemp=initial.temp$X; p0=initial.temp$p0} else {xtemp=Xini; p0=NULL}  #no initial design
+    if(is.null(Xini)){initial.temp=EW_design_initial_MLM(k.continuous=k.continuous, factor.level=factor.level, xlist_fix=xlist_fix, lvec=lvec, uvec=uvec, bvec_matrix=bvec_matrix, link=link, h.func=hfunc, EW_Fi.func=EW_Fi.func, delta0=delta0, epsilon = epsilon, maxit=maxit); xtemp=initial.temp$X; p0=initial.temp$p0} else {xtemp=Xini; p0 = pini}  #no initial design
     if(k.continuous==1) m.design=length(xtemp) else m.design=nrow(xtemp);                   # initial number of design points
     X.mat = rep(0,J*p.factor*m.design);
     dim(X.mat)=c(J, p.factor, m.design)  # initial model matrix X
@@ -109,6 +115,7 @@ EW_ForLion_MLM_Optimal <- function(J ,n.factor, factor.level,var_names=NULL,xlis
     };
 
     if(is.null(p0)){p0 = rep(1/m.design, m.design)}
+    p0 = p0/sum(p0)
 
     optemp=EW_liftoneDoptimal_MLM_func(m=m.design, p=p.factor, Xi=X.mat, J=J, thetavec_matrix=bvec_matrix, link = link, reltol=reltol, maxit=maxit, p00=p0, random=random, nram=nram)
     # new point step (for each idiscrete, give d and partial d function, use optim to find max d compare to p)
@@ -514,6 +521,9 @@ EW_ForLion_MLM_Optimal <- function(J ,n.factor, factor.level,var_names=NULL,xlis
           nit=nit+1;                    # number of candidate y searched
         }# end of while(d()>p) loop
 
+        converge.design=(ytemp$convergence==0);  # TRUE or FALSE
+        if(-ytemp$value/p.factor-1 > reltol) converge.design=FALSE;
+
         #compare the new results with the existing results, replace if better
         if(det.design > det.ans){
           m.design.ans=m.design;      #reported num of design points
@@ -522,8 +532,6 @@ EW_ForLion_MLM_Optimal <- function(J ,n.factor, factor.level,var_names=NULL,xlis
           det.ans = det.design #reported optimized determinant
           x.model.ans = X.mat #reported model matrix
           itmax.design=nit;
-          converge.design=(ytemp$convergence==0);  # TRUE or FALSE
-          if(-ytemp$value/p.factor-1 > reltol) converge.design=FALSE;
           convergence.ans=converge.design
         } # end of if condition (change better random points)
 
@@ -536,7 +544,7 @@ EW_ForLion_MLM_Optimal <- function(J ,n.factor, factor.level,var_names=NULL,xlis
   if((k.continuous>0)&&(k.continuous<d.factor)) {
     lvec=uvec=rep(0, k.continuous);     # lower bounds and upper bounds for continuous factors
     for(i in 1:k.continuous) {lvec[i]=min(factor.level[[i]]); uvec[i]=max(factor.level[[i]]);}; #read in continuous covariates boundary
-    if(is.null(Xini)){initial.temp=EW_design_initial_MLM(k.continuous=k.continuous, factor.level=factor.level, xlist_fix=xlist_fix, lvec=lvec, uvec=uvec, bvec_matrix=bvec_matrix, link=link, h.func=hfunc, EW_Fi.func=EW_Fi.func, delta0=delta0, epsilon = epsilon, maxit=500); xtemp=initial.temp$X; p0=initial.temp$p0} else {xtemp=Xini; p0=NULL}  #no initial design
+    if(is.null(Xini)){initial.temp=EW_design_initial_MLM(k.continuous=k.continuous, factor.level=factor.level, xlist_fix=xlist_fix, lvec=lvec, uvec=uvec, bvec_matrix=bvec_matrix, link=link, h.func=hfunc, EW_Fi.func=EW_Fi.func, delta0=delta0, epsilon = epsilon, maxit=500); xtemp=initial.temp$X; p0=initial.temp$p0} else {xtemp=Xini; p0 = pini}  #no initial design
 
     ## update on 2022/08/28 change EW_Xw_discrete_self function to sequentially and randomly choose x_i^0 => design.initial.self function
 
@@ -548,6 +556,7 @@ EW_ForLion_MLM_Optimal <- function(J ,n.factor, factor.level,var_names=NULL,xlis
       X.mat[,,i]=htemp;
     };
     if(is.null(p0)){p0 = rep(1/m.design, m.design)}
+    p0 = p0/sum(p0)
 
     optemp=EW_liftoneDoptimal_MLM_func(m=m.design, p=p.factor, Xi=X.mat, J=J, thetavec_matrix=bvec_matrix, link=link, reltol=reltol, maxit=maxit, p00=p0, random=random, nram=nram)
 
@@ -1008,10 +1017,8 @@ EW_ForLion_MLM_Optimal <- function(J ,n.factor, factor.level,var_names=NULL,xlis
           nit=nit+1;                    # number of candidate y searched
         }# end of while(d()>p) loop
 
-        itmax.design=nit;
         converge.design=(ytempstar$convergence==0);  # TRUE or FALSE
         if(-ytempstar$value/p.factor-1 > reltol) converge.design=FALSE;
-        convergence.ans=converge.design
         #compare the new results with the existing results, replace if better
         if(det.design > det.ans){
           m.design.ans=m.design;      #reported num of design points
@@ -1020,8 +1027,6 @@ EW_ForLion_MLM_Optimal <- function(J ,n.factor, factor.level,var_names=NULL,xlis
           det.ans = det.design #reported optimized determinant
           x.model.ans = X.mat #reported model matrix
           itmax.design=nit;
-          converge.design=(ytempstar$convergence==0);  # TRUE or FALSE
-          if(-ytempstar$value/p.factor-1 > reltol) converge.design=FALSE;
           convergence.ans=converge.design
         } # end of if condition (change better random points)
 
