@@ -10,5 +10,5 @@
 #'
 
 nu_identity_self <- function(x) {     # for identity link
-  x;
+  1;
 }
